import 'package:flutter/widgets.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart';
import 'package:flutterskel/resources/fonts/FontResources.dart';
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart';
import 'package:flutterskel/resources/themes/ThemeResources.dart';
import 'package:flutterskel/routing/page/PageRouter.dart';
import 'package:flutterskel/routing/url/UrlRouter.dart';
import 'package:flutterskel/userinformation/UserInformationRepository.dart';
import 'package:flutterskel/configuration/ConfigurationRepository.dart' as config;
import 'package:provider/provider.dart';

/// abstract State class
/// This class should only be extended by an abstract class within the app, which fills in all the info except [StateWidget]
/// [StateWidget] should be filled in by the final StatefulWidget class
abstract class RawBaseStatefulWidgetState<
                  StateWidget extends StatefulWidget,
                  // Data
                  DataSource,
                  // Routing
                  URouter extends UrlRouter,
                  PRouter extends PageRouter,
                  // Configuration
                  ConfigurationRepository extends config.ConfigurationRepository,
                  Configuration,
                  // User Information
                  UserRepo extends UserInformationRepository,
                  // Strings
                  StringRepo extends StringsRepository,
                  StringsClass,
                  // Themes (Fonts/Dimensions/Colors)
                  ThemeRepo extends ThemeRepository,
                  Theme extends ThemeResources,
                  Colors extends ColorResources,
                  Dimensions,
                  Fonts extends FontResources,
                  // Images and Icons
                  Images,
                  Icons> extends State<StateWidget> {

  DataSource get dataSource => Provider.of<DataSource>(context);

  URouter get urlRouter => Provider.of<URouter>(context);

  PRouter get pageRouter => Provider.of<PRouter>(context);

  Images get images => Provider.of<Images>(context);

  // Configuration
  ConfigurationRepository get configurationRepository => Provider.of<ConfigurationRepository>(context);
  Configuration get configuration => configurationRepository.configuration;

  UserRepo get userInformation => Provider.of<UserRepo>(context);

  StringsClass get strings => Provider.of<StringRepo>(context).strings;

  ThemeRepo get themes => Provider.of<ThemeRepo>(context);

  Theme get theme => Provider.of<Theme>(context);

  Colors get colors => Provider.of<Colors>(context);

  Dimensions get dimensions => Provider.of<Dimensions>(context);

  Fonts get fonts => Provider.of<Fonts>(context);

  Icons get icons => Provider.of<Icons>(context);
}
